Imports SolidWorks.Interop.sldworks
Imports SolidWorks.Interop.swconst
Imports System.Math


Public Class MeasureClass
    Public Event DoneWithMeasure()

    Dim WithEvents myswSession As SldWorks
    Dim WithEvents ListenerPartDoc As PartDoc
    Dim WithEvents ListenerAssyDoc As AssemblyDoc
    Dim swSelMgr As SelectionMgr
    Dim swMeasure As Measure
    Dim LastLen As Double
    Dim WasLen As Boolean

    Dim SF As Double  '1000 converts to mm, 100/2.54 will convert to inches.
    Dim ANGSF As Double ' 45 * Atn(1) Convert radians to degrees.  Cannot use Atn() function in constant. Value set in InitMeasure.
    Const DISTDEC As Long = 3
    Dim DISTSYM As String
    Const ANGDEC As Long = 2  'Number of "#" is number of decimals
    Const ANGSYM As String = "" 'degrees symbol
    Const MESSAGETIME As Double = 10  'time to display message in seconds

    Public Sub InitMeasure(ByVal mySW As SldWorks)

        myswSession = mySW
        ANGSF = CDbl(45) / Atan(CDbl(1))
        Call myswSession_ActiveDocChangeNotify()

    End Sub

    Private Function myswSession_ActiveDocChangeNotify() As Integer Handles myswSession.ActiveDocChangeNotify
        If False Then 'MsgBox("Exit measure?", MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
            Call I_Want_Out()
        Else
            Dim TmpDoc As ModelDoc2
            TmpDoc = myswSession.ActiveDoc
            If Not (TmpDoc Is Nothing) Then
                If TmpDoc.GetType = SolidWorks.Interop.swconst.swDocumentTypes_e.swDocPART Then
                    ListenerPartDoc = TmpDoc
                    ListenerAssyDoc = Nothing
                    'MsgBox "This is a part"
                ElseIf TmpDoc.GetType = swDocumentTypes_e.swDocDRAWING Then
                    ListenerPartDoc = Nothing
                    ListenerAssyDoc = Nothing
                    'MsgBox "This is a dwg"
                ElseIf TmpDoc.GetType = swDocumentTypes_e.swDocASSEMBLY Then
                    ListenerPartDoc = Nothing
                    ListenerAssyDoc = TmpDoc
                    'MsgBox "This is an assy"
                Else
                    ListenerPartDoc = Nothing
                    ListenerAssyDoc = Nothing
                    swSelMgr = Nothing
                    'MsgBox "none"
                End If
                swSelMgr = TmpDoc.SelectionManager
                swMeasure = TmpDoc.Extension.CreateMeasure
                Select Case TmpDoc.GetUserPreferenceIntegerValue(swUserPreferenceIntegerValue_e.swUnitsLinear)
                    Case swLengthUnit_e.swMM
                        SF = 1000.0#
                        DISTSYM = "mm"
                    Case swLengthUnit_e.swCM
                        SF = 100.0#
                        DISTSYM = "cm"
                    Case swLengthUnit_e.swMETER
                        SF = 1
                        DISTSYM = "m"
                    Case swLengthUnit_e.swINCHES
                        SF = 1000.0# / 25.4
                        DISTSYM = """"
                    Case swLengthUnit_e.swFEET
                        SF = 1000.0# / 25.4 / 12.0#
                        DISTSYM = "'"
                    Case swLengthUnit_e.swFEETINCHES
                        SF = 1000.0# / 25.4
                        DISTSYM = """"
                    Case swLengthUnit_e.swMICRON
                        SF = 1000000.0#
                        DISTSYM = "m"
                    Case swLengthUnit_e.swNANOMETER
                        SF = 1000000000.0#
                        DISTSYM = "nm"
                    Case swLengthUnit_e.swANGSTROM
                        SF = 10000000000.0#
                        DISTSYM = ""
                    Case swLengthUnit_e.swMIL
                        SF = 1000.0# / 25.4 * 1000.0#
                        DISTSYM = "mil"
                    Case swLengthUnit_e.swUIN
                        SF = 1000.0# / 25.4 * 1000000.0#
                        DISTSYM = "in"
                End Select
            End If
            TmpDoc = Nothing
        End If
    End Function

    Private Sub MeasureIt()
        Dim MeasureOK As Boolean
        Dim WriteStr As String
        Dim Dist As Double
        Dim MinDist As Double
        Dim MaxDist As Double
        WriteStr = ""
        swMeasure.ArcOption = swMeasureArcCircleOption_e.swMeasureArcCircle_CenterToCenter
        swMeasure.AngleDecimalPlaces = 16
        MeasureOK = swMeasure.Calculate(Nothing)
        If Not (False = MeasureOK) Then
            'normal distance
            If swMeasure.IsParallel And (swMeasure.NormalDistance <> -1) Then
                WriteStr = "Norm Dist: " & FixNumFmt(swMeasure.NormalDistance * SF, DISTDEC) & DISTSYM
            ElseIf swMeasure.IsParallel And (swMeasure.NormalDistance = -1) Then
                WriteStr = "Norm Dist: " & FixNumFmt(0 * SF, DISTDEC) & DISTSYM
            ElseIf swMeasure.NormalDistance > 0 Then
                WriteStr = "Norm Dist: " & FixNumFmt(swMeasure.NormalDistance * SF, DISTDEC) & DISTSYM
            End If
            'closest distance
            Dist = swMeasure.Distance
            If Dist <> swMeasure.NormalDistance Then
                If Len(WriteStr) > 0 Then
                    WriteStr = WriteStr & " | "
                End If
                WriteStr = WriteStr & "Dist: " & FixNumFmt(Dist * SF, DISTDEC) & DISTSYM
            End If
            'minimum distance
            swMeasure.ArcOption = swMeasureArcCircleOption_e.swMeasureArcCircle_MinimumDistance
            swMeasure.Calculate(Nothing)
            MinDist = swMeasure.Distance
            'maximum distance
            swMeasure.ArcOption = swMeasureArcCircleOption_e.swMeasureArcCircle_MaximumDistance
            swMeasure.Calculate(Nothing)
            MaxDist = swMeasure.Distance
            If MinDist <> Dist Then
                WriteStr = WriteStr & " | min: " & FixNumFmt(MinDist * SF, DISTDEC) & DISTSYM
            End If
            If (MaxDist <> Dist) And (MaxDist <> MinDist) Then
                WriteStr = WriteStr & " | MAX: " & FixNumFmt(MaxDist * SF, DISTDEC) & DISTSYM
            End If
            'Diameter
            If swMeasure.Diameter <> -1 Then
                If Len(WriteStr) > 0 Then
                    WriteStr = WriteStr & " | "
                End If
                WriteStr = WriteStr & "" & FixNumFmt(swMeasure.Diameter * SF, DISTDEC) & DISTSYM
            ElseIf swMeasure.Radius <> -1 Then
                If Len(WriteStr) > 0 Then
                    WriteStr = WriteStr & " | "
                End If
                WriteStr = WriteStr & "" & FixNumFmt(swMeasure.Radius * SF * 2, DISTDEC) & DISTSYM
                WriteStr = WriteStr & " (R " & FixNumFmt(swMeasure.Radius * SF, DISTDEC) & DISTSYM & ")"
            End If
            'length ratio
            If (swMeasure.TotalLength <> -1) And WasLen Then
                If Len(WriteStr) > 0 Then
                    WriteStr = WriteStr & " | "
                End If
                WriteStr = WriteStr & "Ratio: " & FixNumFmt(LastLen / (swMeasure.TotalLength - LastLen), 3)
            End If
            'length
            If swMeasure.Length <> -1 Then
                WasLen = True
                LastLen = swMeasure.Length
                If Len(WriteStr) > 0 Then
                    WriteStr = WriteStr & " | "
                End If
                WriteStr = WriteStr & "Len: " & FixNumFmt(swMeasure.Length * SF, DISTDEC) & DISTSYM
            Else
                WasLen = False
            End If
            'Angle
            If swMeasure.Angle <> -1 Then
                If Len(WriteStr) > 0 Then
                    WriteStr = WriteStr & " | "
                End If
                WriteStr = WriteStr & "Ang: " & FixNumFmt(swMeasure.Angle * ANGSF, ANGDEC) & ANGSYM
            End If
            myswSession.ActiveDoc.Extension.ShowSmartMessage(WriteStr, 1000 * MESSAGETIME, True, True)
        End If
    End Sub

    Private Function ListenerPartDoc_NewSelectionNotify() As Integer Handles ListenerPartDoc.NewSelectionNotify
        If (swSelMgr.GetSelectedObjectCount2(-1) = 1) Or (swSelMgr.GetSelectedObjectCount2(-1) = 2) Then MeasureIt()
    End Function

    Private Function ListenerAssyDoc_NewSelectionNotify() As Integer Handles ListenerAssyDoc.NewSelectionNotify
        If (swSelMgr.GetSelectedObjectCount2(-1) = 1) Or (swSelMgr.GetSelectedObjectCount2(-1) = 2) Then MeasureIt()
    End Function
    Private Function FixNumFmt(ByVal NumIn As Single, ByVal NumDec As Long) As String
        FixNumFmt = Format(NumIn, "0." & New String("#", NumDec))
        If Len(FixNumFmt) < Len(CStr(NumIn)) Then
            'MsgBox("|" & CStr(NumIn) & "|" & Len(CStr(NumIn)) & "|" & FixNumFmt & "|" & Len(FixNumFmt))
            FixNumFmt = FixNumFmt & ""
        End If
    End Function
    Private Sub I_Want_Out()
        'myswSession = Nothing
        'ListenerPartDoc = Nothing
        'ListenerAssyDoc = Nothing
        'swMeasure = Nothing
        'swSelMgr = Nothing
        Me.Finalize()
        RaiseEvent DoneWithMeasure()
        MsgBox("Raised done event")
    End Sub
End Class
